#include  "tilm4f120hqr.h"
#include "serial.h"

// Defines a new version of puts: e(mbedded)puts and egets
// Similar to puts and gets in standard C however egets checks the size
// of the input buffer.  This could be extended to include a timeout quite easily.
// Written by Frank Duignan
// 

// define the size of the communications buffer (adjust to suit)
#define MAXBUFFER   16
typedef struct tagComBuffer{
    unsigned char Buffer[MAXBUFFER];
    volatile int Head,Tail;
    volatile int Count;
} ComBuffer;

ComBuffer ComRXBuffer,ComTXBuffer;

int PutBuf(ComBuffer  *Buf,unsigned char Data);
unsigned char GetBuf(ComBuffer  *Buf);
int GetBufCount(ComBuffer  *Buf);
int ReadCom(int Max,unsigned char *Buffer);
int WriteCom(int Count,const char *Buffer);


void usart_tx (void);
void usart_rx (void);
unsigned ComOpen;
unsigned ComError;
unsigned ComBusy;


int ReadCom(int Max,unsigned char *Buffer)
{
// Read up to Max bytes from the communications buffer
// into Buffer.  Return number of bytes read
	unsigned i;
  	if (!ComOpen)
    	return (-1);
	i=0;
	while ((i < Max-1) && (GetBufCount(&ComRXBuffer)))
		Buffer[i++] = GetBuf(&ComRXBuffer);
	if (i>0)
	{
		Buffer[i]=0;
		return(i);
	}
	else {
		return(0);
	}	
};
int WriteCom(int Count,const char *Buffer)
{
// Writes Count bytes from Buffer into the the communications TX buffer
// returns -1 if the port is not open (configured)
// returns -2 if the message is too big to be sent
// If the transmitter is idle it will initiate interrupts by 
// writing the first character to the hardware transmit buffer
	int i,BufLen;

	if (!ComOpen)
		return (-1);
	do {
		BufLen = GetBufCount(&ComTXBuffer);
	} while ( (MAXBUFFER - BufLen) < Count );
	
	for(i=0;i<Count;i++)
		PutBuf(&ComTXBuffer,Buffer[i]);
	
	if ( (UART0_CTL & BIT8)==0)
	{ // transmitter was idle, turn it on and force out first character
	  SET_BIT(UART0_CTL,BIT8);
	  UART0_DR = GetBuf(&ComTXBuffer);		
	} 
	return 0;
};


void initUART(int BaudRate) {
	int BaudRateDivisor;
	int Delay;
	disable_interrupts();
	ComRXBuffer.Head = ComRXBuffer.Tail = ComRXBuffer.Count = 0;
	ComTXBuffer.Head = ComTXBuffer.Tail = ComTXBuffer.Count = 0;
	ComOpen = 1;
	ComError = 0;
// Turn on the clock for GPIOA (uart 0 uses it) - not sure if I need this
	SET_BIT(SYSCTL_RCGC2,BIT0); 		 // turn on GPIOA
	SET_BIT(SYSCTL_GPIOHBCTL,BIT0);      // turn on AHB access to GPIOA	
// Turn on the clock for the UART0 peripheral	
	SYSCTL_RCGCUART |= BIT0;

// Ensure alternate function number for PA0 and PA1
	SET_BIT(GPIOA_AFSEL,BIT0+BIT1);
	SET_BIT(GPIOA_PUR,BIT0+BIT1);
	SET_BIT(GPIOA_DEN,BIT0+BIT1);
	BaudRateDivisor = 80000000;                // assuming 16MHz clock 
	BaudRateDivisor = BaudRateDivisor / (16 * BaudRate);
	
	UART0_IBRD = BaudRateDivisor;  

	UART0_LCRH = BIT6+BIT5; // no parity, 8 data bits, 1 stop bit
	
	UART0_CTL = BIT9+BIT0; // enable rx and uart

// Enable UART0 interrupts in NVIC	
	SYS_EN0 = BIT5;
// and enable interrupts 
	SET_BIT(UART0_IM,BIT4+BIT5);
	enable_interrupts();
	// The delay below seems to be necessary to allow UART
	// start up properly (inserted after some experimentation)
	Delay=1000;
	while(Delay--);
}
void isr_uart(void) 
{
// check which interrupt happened.
      if (UART0_MIS & BIT5) // is it a TXE interrupt?
		usart_tx();
	if (UART0_MIS & BIT4) // is it an RXNE interrupt?
		usart_rx();
}
void usart_rx (void)
{
// Handles serial comms reception
// simply puts the data into the buffer and sets the ComError flag
// if the buffer is fullup

	if (PutBuf(&ComRXBuffer,(UART0_DR & 0xff)) )
		ComError = 1; // if PutBuf returns a non-zero value then there is an error
	UART0_ICR = BIT4; // clear the rx interrupt
}


void usart_tx (void)
{
// Handles serial comms transmission
// When the transmitter is idle, this is called and the next byte
// is sent (if there is one)
	//SET_BIT(GPIOF_DATA,BIT1+BIT2+BIT3);

	if (GetBufCount(&ComTXBuffer))
	{		
		UART0_DR=GetBuf(&ComTXBuffer);
	}
	else
	{
	  // No more data, disable the transmitter 
	  CLEAR_BIT(UART0_CTL,BIT8);
	}
	UART0_ICR = BIT5; // clear the tx interrupt
	
}
int PutBuf(ComBuffer *Buf,unsigned char Data)
{
	if ( (Buf->Head==Buf->Tail) && (Buf->Count!=0))
		return(1);  /* OverFlow */
	disable_interrupts();
	Buf->Buffer[Buf->Head++] = Data;
	Buf->Count++;
	if (Buf->Head==MAXBUFFER)
		Buf->Head=0;
	enable_interrupts();
	return(0);
};
unsigned char GetBuf(ComBuffer *Buf)
{
    unsigned char Data;
    if ( Buf->Count==0 )
		return (0);
    disable_interrupts();
    Data = Buf->Buffer[Buf->Tail++];
    if (Buf->Tail == MAXBUFFER)
		Buf->Tail = 0;
    Buf->Count--;
    enable_interrupts();
    return (Data);
};
int GetBufCount(ComBuffer *Buf)
{
    return Buf->Count;
};
int eputs(const char *s)
{
  // only writes to the comms port at the moment
  if (!ComOpen)
    return -1;
  // wait if buffer is full
  while( (MAXBUFFER-ComTXBuffer.Count)<=0);
  while (*s) 
    WriteCom(1,s++);
  return 0;
}
int egets(char *s,int Max)
{
  // read from the comms port until end of string
  // or newline is encountered.  Buffer is terminated with null
  // returns number of characters read on success
  // returns 0 or -1 if error occurs
  // Warning: This is a blocking call.
  int Len;
  char c;
  if (!ComOpen)
    return -1;
  Len=0;
  c = 0;
  while ( (Len < Max-1) && (c != NEWLINE) )
  {   
    while (!GetBufCount(&ComRXBuffer)); // wait for a character
    c = GetBuf(&ComRXBuffer);
    s[Len++] = c;
  }
  if (Len>0)
  {
    s[Len]=0;
  }	
  return Len;
}
char HexDigit(int Value)
{
	if ((Value >=0) && (Value < 10))
		return Value+'0';
	else if ((Value >9) && (Value < 16))
		return Value-10 + 'A';
	else
		return 'z';
}

void eputi(int x)
{
	// Output the number over the serial port as
	// as hexadecimal string.
	char TxString[9];
	int Index=8;
	TxString[Index]=0; // terminate the string
	Index--;
	while(Index >=0)
	{
		TxString[Index]=HexDigit(x & 0x0f);
		x = x >> 4;
		Index--;
	}
	eputs(TxString);
}
